/*
BeGLUT v 0.1
by Bob Rost 2001-2002
Minimal GLUT implementation for hardware accelerated OpenGL in BeOS.
This code is in the public domain. Distribute freely.
*/

#include "BeGLUT.h"
#include <stdio.h>
#include <iostream>

BeOS_Hardware_GLUT beosglut_hw; //global state var

/***************************************************************
the BeOS_Hardware_GLUT class implementation
***************************************************************/

BeOS_Hardware_GLUT::BeOS_Hardware_GLUT()
	: BApplication("application/x-vnd.ads.beos_hwgl")
{
	display_callback=NULL;
	idle_callback=NULL;
//	keyboard_callback=NULL;
	motion_callback=NULL;
	passivemotion_callback=NULL;
	mouse_callback=NULL;
	
	double_buffer=false;
	ignore_key_repeat=false;
	window_width=400;
	window_height=300;
	window=NULL;
	main_snooze=1000;
	display_snooze=1000;
}

void BeOS_Hardware_GLUT::Init(int* argc, char** argv)
{
//do nothing
}

void BeOS_Hardware_GLUT::InitDisplayMode(int mode)
{
	if (mode&GLUT_DOUBLE) double_buffer=true;
}

void BeOS_Hardware_GLUT::InitWindowSize(int w, int h)
{
	if(w>0 && h>0)
	{
		window_width=w;
		window_height=h;
	}
}

void BeOS_Hardware_GLUT::InitWindowPosition(int x, int y)
{
}

int BeOS_Hardware_GLUT::EnterGameMode()
{
	return 1;
}

void BeOS_Hardware_GLUT::LeaveGameMode()
{
}


void BeOS_Hardware_GLUT::SetCursor(int cursor)
{
}

void BeOS_Hardware_GLUT::CreateWindow(const char *title)
{
	if (! (window_width&&window_height)) return;
	window=new BeOS_GLUT_Window(title, (float)window_width,
		(float)window_height, double_buffer, display_callback,
		mouse_callback, motion_callback, passivemotion_callback, reshape_callback,
		ignore_key_repeat);
	window->display_snooze=display_snooze;
}

void BeOS_Hardware_GLUT::DestroyWindow(int which)
{
	if(!window) return;
	window->Lock();
	window->QuitRequested();
	delete window;
	//printf("window destroyed\n");fflush(stdout);
}

int BeOS_Hardware_GLUT::GetWindow()
{
	return 1;
}

void BeOS_Hardware_GLUT::IdleFunc(void (*function)(void))
{
	idle_callback=function;
}

void BeOS_Hardware_GLUT::DisplayFunc(void (*function)(void))
{
	display_callback=function;
	if (window) window->display_callback=function;
}

void BeOS_Hardware_GLUT::MouseFunc(void (*function)(int,int,int,int))
{
	mouse_callback=function;
	if (window) window->mouse_callback=function;
}

void BeOS_Hardware_GLUT::MotionFunc(void (*function)(int,int))
{
	motion_callback=function;
	if (window) window->motion_callback=function;
}

void BeOS_Hardware_GLUT::PassiveMotionFunc(void (*function)(int,int))
{
	passivemotion_callback=function;
	if (window) window->passivemotion_callback=function;
}

void BeOS_Hardware_GLUT::ReshapeFunc(void(*function)(int,int))
{
	reshape_callback=function;
	if(window) window->reshape_callback=function;
}

void BeOS_Hardware_GLUT::MainLoop()
{
	initialized=true;
	if (!window)
	{
		printf("No window. Use glutCreateWindow(const char *title) before glutMainLoop()\n");
		fflush(stdout);
		return;
	}
	glutPostRedisplay();
	while(window && window->initialized)
	{
		//if keypressed then keyboard_callback
		if (idle_callback) idle_callback();
//		cout<<idle_callback<<endl;
		snooze(main_snooze);
	}
	QuitRequested();
}

void BeOS_Hardware_GLUT::SwapBuffers()
{
	if(!window) return;
	if (window->double_buffer)
	{
		window->SwapBuffers();
		glDrawBuffer(GL_BACK);
	}
	else glDrawBuffer(GL_FRONT);
}

void BeOS_Hardware_GLUT::PostRedisplay()
{
	if (window) window->post_redisplay=true;
}

bool BeOS_Hardware_GLUT::QuitRequested()
{
//cout<<"GLUT quit requested"<<endl;
	if(window)
	{
		window->Lock();
		window->QuitRequested();
	}
	initialized=false;
	return true;
}

void BeOS_Hardware_GLUT::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{
		case B_QUIT_REQUESTED:
			//cout<<"msg quit"<<endl;
			initialized=false;
			break;
		default:
			BApplication::MessageReceived(msg);
			break;
	}
}

void BeOS_Hardware_GLUT::ReshapeWindow(int w, int h)
{
	if (window)
		window->ResizeTo((float)w,(float)h);
}

void BeOS_Hardware_GLUT::WarpPointer(int x, int y)
{
	if(!window) return;
	BMessage *msg=new BMessage(B_MOUSE_MOVED);
	msg->AddPoint("where",BPoint((float)x,(float)y));
	window->PostMessage(msg);
}

void BeOS_Hardware_GLUT::IgnoreKeyRepeat(int ignore)
{
	bool val=(ignore==0?false:true);
	ignore_key_repeat=val;
	if (window) window->ignore_key_repeat=val;
}

void BeOS_Hardware_GLUT::BeMainSnooze(unsigned amount)
{
	main_snooze=amount;
}

void BeOS_Hardware_GLUT::BeDisplaySnooze(unsigned amount)
{
	display_snooze=amount;
	if (window) window->display_snooze=amount;
}

void BeOS_Hardware_GLUT::KeyboardFunc(void (*function)(unsigned char, int, int))
{
	if (window) window->keyboard_callback=function;
}

void BeOS_Hardware_GLUT::KeyboardUpFunc(void (*function)(unsigned char, int, int))
{
	if (window) window->keyboard_up_callback=function;
}

void BeOS_Hardware_GLUT::SpecialFunc(void (*function)(int, int, int))
{
	if (window) window->special_callback=function;
}

void BeOS_Hardware_GLUT::SpecialUpFunc(void (*function)(int, int, int))
{
	if (window) window->special_up_callback=function;
}

