/*
BeGLUT v 0.1
by Bob Rost 2001-2002
Minimal GLUT implementation for hardware accelerated OpenGL in BeOS.
This code is in the public domain. Distribute freely.
*/

#ifndef _BEOS_HWGL_GLUT_H_
#define _BEOS_HWGL_GLUT_H_

#include <app/Application.h>
#include <DirectGLWindow.h>
#include <opengl/GL/gl.h>
#include <opengl/GL/glu.h>
#include <support/Locker.h>
#include <kernel/OS.h>

//glut defs
#define GLUT_RGBA 0
#define GLUT_SINGLE 0
#define GLUT_DOUBLE 1
#define GLUT_UP 0
#define GLUT_DOWN 1
#define GLUT_LEFT_BUTTON 1
#define GLUT_RIGHT_BUTTON 2
#define GLUT_MIDDLE_BUTTON 4

//glut functions
#define glutInit(argc,argv)			beosglut_hw.Init(argc,argv)
#define glutInitDisplayMode(mode)	beosglut_hw.InitDisplayMode(mode) //ignored
#define glutInitWindowSize(w,h)		beosglut_hw.InitWindowSize(w,h)
#define glutInitWindowPosition(x,y)	beosglut_hw.InitWindowPosition(x,y) //ignored
#define glutEnterGameMode()			beosglut_hw.EnterGameMode()	//returns 1
#define glutLeaveGameMode()			beosglut_hw.LeaveGameMode() //ignored
#define glutSetCursor(cursor)		beosglut_hw.SetCursor(cursor) //ignored
#define glutDestroyWindow(which)	beosglut_hw.DestroyWindow(which)
#define glutCreateWindow(name)		beosglut_hw.CreateWindow(name)
#define glutGetWindow()				beosglut_hw.GetWindow()
#define glutIdleFunc(function)		beosglut_hw.IdleFunc(function)
#define glutDisplayFunc(function)	beosglut_hw.DisplayFunc(function)
#define glutMainLoop()				beosglut_hw.MainLoop()
#define glutSwapBuffers()			beosglut_hw.SwapBuffers()
#define glutPostRedisplay()			beosglut_hw.PostRedisplay()
#define glutReshapeWindow(w,h)		beosglut_hw.ReshapeWindow(w,h)
#define glutReshapeFunc(function)	beosglut_hw.ReshapeFunc(function)
#define glutMotionFunc(function)	beosglut_hw.MotionFunc(function)
#define glutPassiveMotionFunc(f)	beosglut_hw.PassiveMotionFunc(f)
#define glutMouseFunc(function)		beosglut_hw.MouseFunc(function)
#define glutWarpPointer(x,y)		beosglut_hw.WarpPointer(x,y)
#define glutIgnoreKeyRepeat(val)	beosglut_hw.IgnoreKeyRepeat(val)
#define glutKeyboardFunc(function)	beosglut_hw.KeyboardFunc(function)
#define glutKeyboardUpFunc(func)	beosglut_hw.KeyboardUpFunc(func)
#define glutSpecialFunc(function)	beosglut_hw.SpecialFunc(function)
#define glutSpecialUpFunc(func)		beosglut_hw.SpecialUpFunc(func)

//beos glut functions
#define beglutMainSnooze(amount)	beosglut_hw.BeMainSnooze(amount)
#define beglutDisplaySnooze(amount)	beosglut_hw.BeDisplaySnooze(amount)

/////////////////////////////////////////////////////////////////////////
//definitions
class BeOS_Hardware_GLUT;
class BeOS_GLUT_Window;
extern BeOS_Hardware_GLUT beosglut_hw; //global state var

//class declarations
class BeOS_Hardware_GLUT : public BApplication
{
	public:
		BeOS_Hardware_GLUT();
		//these first functions are here for compatibility only and do nothing
		void InitWindowPosition(int x, int y);
		int EnterGameMode();
		void LeaveGameMode();
		void Init(int* argc, char** argv);
		void SetCursor(int cursor);
		
		//here are actually implemented functions
		void InitDisplayMode(int mode);
		void InitWindowSize(int w, int h);
		void CreateWindow(const char* name);
		void DestroyWindow(int which);
		int GetWindow(); //returns 1
		void KeyboardFunc(void (*function)(unsigned char, int, int));
		void KeyboardUpFunc(void (*function)(unsigned char, int, int));
		void SpecialFunc(void (*function)(int,int,int));
		void SpecialUpFunc(void (*function)(int,int,int));
		void IdleFunc(void (*function)(void));
		void DisplayFunc(void (*function)(void));
		void MainLoop();
		void SwapBuffers();
		void PostRedisplay();
		void ReshapeWindow(int w, int h);
		void MotionFunc(void(*function)(int x, int y));
		void PassiveMotionFunc(void(*function)(int x, int y));
		void MouseFunc(void (*function)(int button, int state, int x, int y));
		void ReshapeFunc(void (*function)(int width, int height));
		void WarpPointer(int x, int y);
		void IgnoreKeyRepeat(int ignore);
		virtual bool QuitRequested();
		virtual void MessageReceived(BMessage*);
		void BeMainSnooze(unsigned amount);
		void BeDisplaySnooze(unsigned amount);
	private:
		void (*display_callback)(void);
		void (*idle_callback)(void);
		void (*motion_callback)(int,int);
		void (*passivemotion_callback)(int,int);
		void (*mouse_callback)(int,int,int,int);
		void (*reshape_callback)(int,int);
		bool double_buffer;
		int window_width;
		int window_height;
		BeOS_GLUT_Window *window;
		bool initialized;
		bool ignore_key_repeat;
		unsigned main_snooze; //default 1000
		unsigned display_snooze; //default 1000
};

#define GLUT_MSG_FULLSCREEN 'full'
#define GLUT_MAX_DEVICES 16
typedef struct ModeInfoRec ModeInfo;
struct ModeInfoRec
{
	ModeInfo *next;
	uint32 xres;
	uint32 yres;
};

typedef struct GfxDeviceRec
{
	uint32 deviceID;
	ModeInfo *first;
} GfxDevice;


int32 beos_glut_render_thread( void *data );

class BeOS_GLUT_Window : public BDirectGLWindow
{
public:
	BeOS_GLUT_Window( const char*,float,float,bool,void(*drawfunc)(void),
		void(*mousefunc)(int,int,int,int),
		void(*motionfunc)(int,int),void(*passivemotionfunc)(int,int),
		void(*reshapefunc)(int,int),bool ignorerepeat);
	virtual ~BeOS_GLUT_Window( void );
	virtual bool QuitRequested( void );
	virtual void MessageReceived( BMessage *msg );
	virtual void DeviceInfo( uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum );
	virtual void VideoModeInfo( uint32 width, uint32 height, uint32 color, uint32 depth, uint32 stencil, uint32 accum );
	virtual void FrameResized(float,float);
	void NoPostRedisplay();
	
	unsigned display_snooze;
	bool window_teardown;
	bool post_redisplay;
	bool double_buffer;
	void (*display_callback)(void);
	void (*motion_callback)(int,int);
	void (*passivemotion_callback)(int,int);
	void (*mouse_callback)(int,int,int,int);
	void (*reshape_callback)(int,int);
	void (*keyboard_callback)(unsigned char,int,int);
	void (*keyboard_up_callback)(unsigned char,int,int);
	void (*special_callback)(int,int,int);
	void (*special_up_callback)(int,int,int);
	bool initialized;
	bool ignore_key_repeat;
	thread_id draw_thread;
private:
	status_t RestartGL( uint32 deviceID );
	static int32 beos_glut_render_thread( void *data );

	BPoint mouse;
	bool fullscreenEnabled;
	int32 fullscreenWidth;
	int32 fullscreenHeight;
	int32 currentDevice;
	GfxDevice devices[GLUT_MAX_DEVICES];
};

#endif