/*
BeGLUT v 0.1
by Bob Rost 2001-2002
Minimal GLUT implementation for hardware accelerated OpenGL in BeOS.
This code is in the public domain. Distribute freely.
*/

#include "BeGLUT.h"
#include <stdio.h>
#include <View.h>
/***************************************************************
the BeOS_GLUT_Window class BDirectGLWindow implementation
***************************************************************/

int32 BeOS_GLUT_Window::beos_glut_render_thread( void *data )
{
	BeOS_GLUT_Window *wnd = (BeOS_GLUT_Window *)data;
	wnd->MakeCurrent();
	while( !wnd->window_teardown )
	{
		if (wnd->post_redisplay)
		{
			wnd->NoPostRedisplay();
			if (wnd->display_callback) wnd->display_callback();
		}
		wnd->YieldCurrent();
		snooze(wnd->display_snooze);
	}
	wnd->ReleaseCurrent();
	return 0;
}

BeOS_GLUT_Window::BeOS_GLUT_Window( const char *title, float w, float h, bool db,
	void(*drawfunc)(void),
	void(*mousefunc)(int,int,int,int), void(*motionfunc)(int,int),
	void(*passivemotionfunc)(int,int),void(*reshapefunc)(int,int),
	bool ignorerepeat)
	:BDirectGLWindow( BRect(100.0,100.0,100.0+w,100+h),title,
					B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
					B_ASYNCHRONOUS_CONTROLS | B_NOT_ZOOMABLE /*| B_NO_RESIZE*/)	
{
	window_teardown=false;
	post_redisplay=false;
	initialized=false;
	double_buffer=db;
	keyboard_callback=NULL;
	keyboard_up_callback=NULL;
	special_callback=NULL;
	special_up_callback=NULL;
	display_callback=drawfunc;
	mouse_callback=mousefunc;
	motion_callback=motionfunc;
	passivemotion_callback=passivemotionfunc;
	reshape_callback=reshapefunc;
	ignore_key_repeat=ignorerepeat;

	draw_thread=-1;
	currentDevice=-1;
	mouse=BPoint(0,0);
	EnumerateDevices( BGL_MONITOR_ANY, BGL_FASTEST | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE );
	if( RestartGL(currentDevice)<B_OK)
		if( RestartGL( BGL_DEVICE_SOFTWARE ) < B_OK ) return;
	Show();
}

BeOS_GLUT_Window::~BeOS_GLUT_Window( void )
{
	status_t ret;
	Hide();
	Sync();
	window_teardown=true;
	wait_for_thread(draw_thread, &ret);
}

bool BeOS_GLUT_Window::QuitRequested( void )
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	window_teardown=true;
	initialized=false;
	Quit();
	return true;
}

void BeOS_GLUT_Window::MessageReceived( BMessage *msg )
{
	status_t ok;
	int32 temp_int,temp_int2;
	int temp2;
	switch(msg->what)
	{
		case B_KEY_UP:
			ok=msg->FindInt32("raw_char",&temp_int);
			if (keyboard_up_callback)
				keyboard_up_callback((uchar)temp_int,(int)mouse.x,(int)mouse.y);
			break;
		case B_KEY_DOWN:
			ok=msg->FindInt32("raw_char",&temp_int);
			if(ignore_key_repeat &&
				(msg->FindInt32("be:key_repeat",&temp_int2)<B_OK))
				break;
			if (keyboard_callback)
				keyboard_callback((uchar)temp_int,(int)mouse.x,(int)mouse.y);
			break;
		case B_MOUSE_MOVED:
			msg->FindPoint("where",&mouse);
			msg->FindInt32("buttons",&temp_int);
			if(temp_int && motion_callback) //drag
				motion_callback((int)mouse.x,(int)mouse.y);
			else if(passivemotion_callback) //passive move
				passivemotion_callback((int)mouse.x,(int)mouse.y);
			break;
		case B_MOUSE_DOWN:
			msg->FindPoint("where",&mouse);
			msg->FindInt32("buttons",&temp_int);
			temp2=0;
			if(temp_int & B_PRIMARY_MOUSE_BUTTON) temp2|=GLUT_LEFT_BUTTON;
			if(temp_int & B_SECONDARY_MOUSE_BUTTON) temp2|=GLUT_RIGHT_BUTTON;
			if(temp_int & B_TERTIARY_MOUSE_BUTTON) temp2|=GLUT_MIDDLE_BUTTON;
			if (mouse_callback)
				mouse_callback(temp2,GLUT_DOWN,(int)mouse.x,(int)mouse.y);
			break;
		case B_MOUSE_UP:
			msg->FindPoint("where",&mouse);
			msg->FindInt32("buttons",&temp_int);
			temp2=0;
			if(temp_int & B_PRIMARY_MOUSE_BUTTON) temp2|=GLUT_LEFT_BUTTON;
			if(temp_int & B_SECONDARY_MOUSE_BUTTON) temp2|=GLUT_RIGHT_BUTTON;
			if(temp_int & B_TERTIARY_MOUSE_BUTTON) temp2|=GLUT_MIDDLE_BUTTON;
			if (mouse_callback)
				mouse_callback(temp2,GLUT_UP,(int)mouse.x,(int)mouse.y);
			break;
		default:
			BDirectWindow::MessageReceived(msg);
			break;
	}
}

status_t BeOS_GLUT_Window::RestartGL( uint32 deviceID )
{
	status_t ret;
	//destroy old draw thread
	window_teardown = true;
	if( draw_thread >= 0 )
		wait_for_thread( draw_thread, &ret );
	//make a new GL setup
	if( initialized )
	{
		ShutdownGL();
		initialized = false;
	}
	initialized = true;
	window_teardown = false;
	// Start the drawing thread.
	draw_thread = spawn_thread( beos_glut_render_thread, "OpenGL drawing thread", B_DISPLAY_PRIORITY, this );
	if( draw_thread < 0 )
	{
		fprintf( stderr, "Can't spawn drawing thread, bailing out...\n" );
		be_app->PostMessage( B_QUIT_REQUESTED );
		return B_ERROR;
	}
	fullscreenEnabled = false;
	if( InitializeGL( deviceID, BGL_FASTEST | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE ) < B_OK )
	{
		printf( "BeOS_GLUT_Window::RestartGL -- Error initializing OpenGL\n" );
		fflush(stdout);
		be_app->PostMessage( B_QUIT_REQUESTED );
		return B_ERROR;
		//exit(0);  // Fix me
	}
	int32 ct;
	for( ct=0; ct<GLUT_MAX_DEVICES; ct++ )
	{
		if( devices[ct].deviceID == deviceID )
			break;
	}
	if( ct < GLUT_MAX_DEVICES )
	{
		//int32 index = 0;
		ModeInfo *mInfo = devices[ct].first;
		while( mInfo )
		{
			if( mInfo->xres )
			{
				char buf[64];
				sprintf( buf, "%dx%d", (unsigned)mInfo->xres, (unsigned)mInfo->yres );
			}
			mInfo = mInfo->next;
		}
	}
	resume_thread( draw_thread );
	return B_OK;
}


void BeOS_GLUT_Window::DeviceInfo( uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum )
{
	//char buf[256];
	if( device_id == BGL_DEVICE_SOFTWARE )
	{
	}
	currentDevice++;
	devices[currentDevice].deviceID = device_id;
	devices[currentDevice].first = 0;
}

void BeOS_GLUT_Window::VideoModeInfo( uint32 width, uint32 height, uint32 color, uint32 depth, uint32 stencil, uint32 accum )
{
	ModeInfo *m = new ModeInfo;
	m->next = 0;
	m->xres = width;
	m->yres = height;
	if( devices[currentDevice].first )
	{
		ModeInfo *last = devices[currentDevice].first;
		while( last->next )
			last = last->next;
		last->next = m;
	}
	else
	{
		devices[currentDevice].first = m;
	}
}

void BeOS_GLUT_Window::NoPostRedisplay()
{
	post_redisplay=false;
}

void BeOS_GLUT_Window::FrameResized( float width, float height )
{
	MakeCurrent();
//	glViewport( 0, 0, (int)width, (int)height );
	if(reshape_callback) reshape_callback((int)width,(int)height);
	ReleaseCurrent();
}
